/**
 * Copyright 2013 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.commands.builtin.internal;

import static org.testng.Assert.assertEquals;

import java.util.Arrays;
import java.util.List;

import org.apache.felix.service.command.CommandSession;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.ow2.shelbie.testing.ActionContainer;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * @author Florent Benoit
 */
public class EchoActionTestCase {

    @Mock
    private CommandSession session;

    private ActionContainer echoActionCommand;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
        echoActionCommand = new ActionContainer(new EchoAction());
    }

    @Test
    public void testHello() throws Exception {
        List<String> lst = Arrays.asList(new String[] {"Hello world"});
        echoActionCommand.argument(0, lst);
        echoActionCommand.execute(session);
        assertEquals(echoActionCommand.getSystemOut(), "Hello world\n");
    }

    @Test
    public void testColorHello() throws Exception {
        List<String> lst = Arrays.asList(new String[] {"Hello @|bold,red world|@"});
        echoActionCommand.argument(0, lst);
        echoActionCommand.execute(session);
        assertEquals(echoActionCommand.getSystemOut(), "Hello [1;31mworld[m\n");
    }

    @Test
    public void testNoTrailingNewline() throws Exception {
        List<String> lst = Arrays.asList(new String[] {"Hello world"});
        echoActionCommand.argument(0, lst);
        echoActionCommand.option("-n", true);
        echoActionCommand.execute(session);
        assertEquals(echoActionCommand.getSystemOut(), "Hello world");
    }



}
