/**
 * Copyright 2012 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.converter;

import org.apache.felix.service.command.Converter;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Locale;
import java.util.Properties;
import java.util.regex.Pattern;

import static org.testng.Assert.assertEquals;

/**
 * Created by IntelliJ IDEA.
 * User: guillaume
 * Date: 08/03/12
 * Time: 21:48
 * To change this template use File | Settings | File Templates.
 */
public class BasicTypesConverterTestCase {

    private static enum Mode {
        IN, OUT
    }
    
    private Converter converter;
    
    @BeforeMethod
    public void setUp() throws Exception {
        converter = new BasicTypesConverter();
    }

    @Test
    public void testSupportedTypeConversion() throws Exception {
        // Class
        assertEquals(converter.convert(Class.class, BasicTypesConverter.class.getName()), BasicTypesConverter.class);

        // Locale
        assertEquals(converter.convert(Locale.class, "fr"), Locale.FRENCH);
        assertEquals(converter.convert(Locale.class, "fr_FR"), Locale.FRANCE);
        assertEquals(converter.convert(Locale.class, "fr_FR_fr"), new Locale("fr", "FR", "fr"));
        
        // Pattern
        Pattern pattern = (Pattern) converter.convert(Pattern.class, ".*");
        assertEquals(pattern.toString(), ".*");
        
        // Properties
        Properties p1 = (Properties) converter.convert(Properties.class, "test=hello");
        assertEquals(p1.getProperty("test"), "hello");
        Properties p2 = (Properties) converter.convert(Properties.class, "test=hello\ntest2=world");
        assertEquals(p2.getProperty("test"), "hello");
        assertEquals(p2.getProperty("test2"), "world");

        // Boolean
        assertEquals(converter.convert(Boolean.class, "on"), Boolean.TRUE);
        assertEquals(converter.convert(Boolean.class, "true"), Boolean.TRUE);
        assertEquals(converter.convert(Boolean.class, "yes"), Boolean.TRUE);
        assertEquals(converter.convert(Boolean.class, "off"), Boolean.FALSE);
        assertEquals(converter.convert(Boolean.class, "false"), Boolean.FALSE);
        assertEquals(converter.convert(Boolean.class, "no"), Boolean.FALSE);

        // Numbers
        assertEquals(converter.convert(Integer.class, "42"), Integer.valueOf(42));
        assertEquals(converter.convert(Short.class, "42"), Short.valueOf("42"));
        assertEquals(converter.convert(Long.class, "42"), Long.valueOf(42));
        assertEquals(converter.convert(Float.class, "3.14159"), Float.valueOf(3.14159f));
        assertEquals(converter.convert(Double.class, "3.14159"), Double.valueOf(3.14159d));
        assertEquals(converter.convert(Byte.class, "2"), Byte.valueOf("2"));

        // Character
        assertEquals(converter.convert(Character.class, "\\u0024"), '$');
        assertEquals(converter.convert(Character.class, "a"), 'a');

        // Enum
        assertEquals(converter.convert(Mode.class, "IN"), Mode.IN);
        assertEquals(converter.convert(Mode.class, "OUT"), Mode.OUT);
    }

    @Test(expectedExceptions = Exception.class)
    public void testInvalidBooleanValue() throws Exception {
        assertEquals(converter.convert(Boolean.class, "invalid"), Boolean.FALSE);
    }

    @Test(expectedExceptions = Exception.class)
    public void testInvalidCharacterValue() throws Exception {
        assertEquals(converter.convert(Character.class, "abcd"), 'a');
    }
}
