/**
 * Copyright 2012 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.converter;

import org.apache.felix.service.command.Converter;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import static org.testng.Assert.assertEquals;

/**
 * @author Guillaume Sauthier
 */
public class NumberConverterTestCase {

    private NumberConverter converter;

    @BeforeMethod
    public void setUp() throws Exception {
        converter = new NumberConverter();
    }

    @Test
    public void testConvertToInteger() throws Exception {
        assertEquals(convert(Integer.class, Double.valueOf(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, Long.valueOf(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, Short.valueOf("1")), Integer.valueOf(1));
        assertEquals(convert(Integer.class, Byte.valueOf("1")), Integer.valueOf(1));
        assertEquals(convert(Integer.class, Float.valueOf(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, BigDecimal.valueOf(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, BigInteger.valueOf(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, new AtomicLong(1)), Integer.valueOf(1));
        assertEquals(convert(Integer.class, new AtomicInteger(1)), Integer.valueOf(1));
    }
    
    @Test
    public void testFormatAllSupportedTypes() throws Exception {
        assertEquals(converter.format(new AtomicLong(42), Converter.PART, null), "42");
        assertEquals(converter.format(BigInteger.valueOf(42), Converter.PART, null), "42");
        assertEquals(converter.format(Double.valueOf(42.2), Converter.PART, null), "42.2");
        assertEquals(converter.format(new AtomicInteger(42), Converter.PART, null), "42");
        assertEquals(converter.format(Long.valueOf(42), Converter.PART, null), "42");
        assertEquals(converter.format(BigDecimal.valueOf(42), Converter.PART, null), "42");
        assertEquals(converter.format(Integer.valueOf(42), Converter.PART, null), "42");
        assertEquals(converter.format(Short.valueOf("42"), Converter.PART, null), "42");
        assertEquals(converter.format(Byte.valueOf("42"), Converter.PART, null), "42");
        assertEquals(converter.format(Float.valueOf(42.2f), Converter.PART, null), "42.2");
    }

    private <T> T convert(Class<T> type, Object in) throws Exception {
        return type.cast(converter.convert(type, in));
    }

}
