/**
 * Copyright 2012 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler;

import org.apache.felix.gogo.commands.Action;
import org.apache.felix.gogo.commands.Argument;
import org.apache.felix.gogo.commands.Command;
import org.apache.felix.gogo.commands.Option;
import org.apache.felix.service.command.CommandSession;
import org.mockito.Mock;
import org.ow2.shelbie.core.IConverterManager;
import org.ow2.shelbie.core.IUsagePrinter;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.PrintStream;

import static java.util.Arrays.asList;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anySetOf;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.initMocks;
import static org.testng.Assert.assertEquals;

/**
 * A {@code GogoPreparatorTestCase} is ...
 *
 * @author Guillaume Sauthier
 */
public class GogoPreparatorTestCase {

    private GogoPreparator preparator;

    @Mock
    private IConverterManager converter;
    
    @Mock
    private IUsagePrinter printer;

    @BeforeMethod
    public void setUp() throws Exception {
        initMocks(this);
        preparator = new GogoPreparator();
        preparator.bindConverterManager(converter);
        preparator.bindPrinter(printer);
    }

    @Test
    public void testPrepareWithHelpOption() throws Exception {
        
        HelloAction action = new HelloAction();
        preparator.prepare(action, null, asList((Object) "--help"));

        Command command = HelloAction.class.getAnnotation(Command.class);

        verify(printer).printUsage(eq(command),
                                   anySetOf(Option.class),
                                   anySetOf(Argument.class),
                                   any(PrintStream.class));
    }
    
    @Test
    public void testPrepare() throws Exception {

        when(converter.convert(eq(String.class), eq("fr"))).thenReturn("fr");
        when(converter.convert(eq(String.class), eq("Guillaume"))).thenReturn("Guillaume");

        HelloAction action = new HelloAction();
        preparator.prepare(action, null, asList((Object) "--lang", "fr", "Guillaume"));

        // Ensure fields have been prepared
        assertEquals(action.lang, "fr");
        assertEquals(action.target, "Guillaume");
    }

    @Command(scope = "test",
             name = "hello")
    private static class HelloAction implements Action {
        
        @Option(name = "--lang",
                required = true)
        String lang;
        
        @Argument
        String target;

        public Object execute(CommandSession session) throws Exception {
            return null;
        }
    }

}
