/**
 * Copyright 2013 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.history;

import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.osgi.framework.BundleContext;
import org.ow2.shelbie.core.identity.IdentityProvider;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.security.auth.Subject;
import java.io.File;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * Created with IntelliJ IDEA.
 * User: guillaume
 * Date: 07/02/13
 * Time: 18:25
 * To change this template use File | Settings | File Templates.
 */
public class DefaultHistoryFileProviderTestCase {

    @Mock
    private BundleContext m_bundleContext;

    @Mock
    private IdentityProvider m_identityProvider;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testGetHistoryFile() throws Exception {
        Subject subject = new Subject();
        File marker = new File(getClass().getResource("/marker.txt").toURI());
        File base = marker.getParentFile();
        when(m_bundleContext.getDataFile("histories")).thenReturn(base);
        when(m_identityProvider.get(subject)).thenReturn("guillaume");

        DefaultHistoryFileProvider provider = new DefaultHistoryFileProvider(m_bundleContext);
        provider.bindIdentityProvider(m_identityProvider);
        File history = provider.getHistoryFile(subject);

        assertEquals(history.getName(), "guillaume.history");
        assertEquals(history.getParentFile(), base);
    }
}
